@@ -1,7 +1,7 @@
 /* srs-speech.c
- *
- * Copyright 2001, 2002 Sun Microsystems, Inc.,
- * Copyright 2001, 2002 BAUM Retec, A.G.
+ * 
+ * Copyright 2001 - 2005 Sun Microsystems, Inc.,
+ * Copyright 2001 - 2005 BAUM Retec, A.G.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -378,11 +378,12 @@ srs_speak_text_out (SRSTextOut *tout)
 	    index = srs_sp_letter_get_index_for_spell (letter);
 
 	    text = g_string_new ("");
-/*	    if (g_unichar_isupper (letter)) 
+	    if (g_unichar_isupper (letter)) 
 	    {
-		g_string_append (text, _("upper"));
+		/* To translators: this should be as short as possible */
+		g_string_append (text, _("cap"));
 		g_string_append (text, " ");
-	    }*/
+	    }
 	    if (index >= 0)
 		g_string_append (text,
 			    tout->spelling == SRS_SPELLING_CHAR ? 
