@@ -1,7 +1,7 @@
 /* srbrl.c
  *
- * Copyright 2001, 2002 Sun Microsystems, Inc.,
- * Copyright 2001, 2002 BAUM Retec, A.G.
+ * Copyright 2001 - 2005 Sun Microsystems, Inc.,
+ * Copyright 2001 - 2005 BAUM Retec, A.G.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -54,7 +54,7 @@ typedef struct
     gchar 	 *cursor_style;	
     gchar 	 *translation_table;
     
-    gint  	 port;
+    gchar  	 *port;
     gint  	 position_sensor;
     gint  	 optical_sensor;
     gint  	 offset;
@@ -641,20 +641,18 @@ src_braille_translation_table_exist (const gchar *table)
 gboolean
 src_braille_restart ()
 {
+    gchar *device = g_strdup (src_braille->device);
+    gchar *port = g_strdup (src_braille->port);
+
     if (src_use_braille)
-    {
-	gchar *device = g_strdup (src_braille->device);
-	gint port = src_braille->port;
-	/* terminate Braille */
 	src_braille_terminate ();
     	/* and then start it again */
-	src_braille->device = device;
-	src_braille->port = port;	
-	src_use_braille = src_braille_init ();
-    }
-    if (src_use_braille_monitor)
-    {
-    }
+	
+    src_braille->device = device;
+    src_braille->port = port;	
+
+    src_use_braille = src_braille_init ();
+
     return src_use_braille;
 }
 
@@ -759,9 +757,61 @@ src_braille_setting_new ()
     return braille;
 }
 
+static gchar*
+src_braille_get_port_from_no (gint port_no)
+{
+    gchar *port = NULL;
+
+    if (MIN_SERIAL_PORT_NO > port_no ||
+	    MAX_SERIAL_PORT_NO < port_no) 
+    {
+	port_no = DEFAULT_BRAILLE_PORT_NO;
+    	SET_BRAILLE_CONFIG_DATA (BRAILLE_PORT_NO, CFGT_INT, &port_no);
+    }   
+    switch (port_no)
+    {
+	case -1: port = NULL; 	break;
+	case  0: port = g_strdup (TTYNAME0); 	break;
+	case  1: port = g_strdup (TTYNAME1);	break;
+	case  2: port = g_strdup (TTYNAME2); 	break;
+	case  3: port = g_strdup (TTYNAME3);	break;
+	default:
+	    fprintf (stderr, "\nbrl_open_port: Invalid serial port number %d", port_no);
+	    break;
+    }
+    if (!port && DEFAULT_BRAILLE_PORT)
+	port = g_strdup (DEFAULT_BRAILLE_PORT);
+
+    return port;
+}
+
+
+static gchar*
+src_braille_get_port ()
+{
+    gchar *port = DEFAULT_BRAILLE_PORT;
+
+    GET_BRAILLE_CONFIG_DATA_WITH_DEFAULT (BRAILLE_PORT, CFGT_STRING, 
+					      &port, &port);
+    if (!port || !port[0])
+    {
+	gint port_no = DEFAULT_BRAILLE_PORT_NO;
+        GET_BRAILLE_CONFIG_DATA_WITH_DEFAULT (BRAILLE_PORT_NO, CFGT_INT, 
+					      &port_no, &port_no);
+	port = src_braille_get_port_from_no (port_no);
+    }
+    return port;
+}
+
 static void
 src_braille_load_values (SRCBraille *braille)
 {
+    gchar *default_braille_transaltion_table;
+    gchar *default_braille_style;
+    gchar *default_braille_cursor_style;
+    gint default_pos_sensor;
+    gint default_opt_sensor;
+
     sru_assert (braille);
     
     if (!braille->device)
@@ -771,24 +821,10 @@ src_braille_load_values (SRCBraille *braille)
 					      &braille->device, 
 					      default_device);
     }
-        
-    if (MIN_BRAILLE_PORT > braille->port || 
-	MAX_BRAILLE_PORT < braille->port)
-    {
-	gint default_port = DEFAULT_BRAILLE_PORT_NO;
-	GET_BRAILLE_CONFIG_DATA_WITH_DEFAULT (BRAILLE_PORT_NO, CFGT_INT, 
-					      &braille->port, 
-					      &default_port);
+    g_free (braille->port);
+    braille->port = src_braille_get_port ();
     
-	if (MIN_BRAILLE_PORT > braille->port || 
-	    MAX_BRAILLE_PORT < braille->port) 
-	{
-    	    braille->port = DEFAULT_BRAILLE_PORT_NO;
-    	    SET_BRAILLE_CONFIG_DATA (BRAILLE_PORT_NO, CFGT_INT, &braille->port);
-	}					  
-    }    
-    
-    gchar *default_braille_transaltion_table = DEFAULT_BRAILLE_TRANSLATION;
+    default_braille_transaltion_table = DEFAULT_BRAILLE_TRANSLATION;
     GET_BRAILLE_CONFIG_DATA_WITH_DEFAULT (BRAILLE_TRANSLATION, CFGT_STRING, 
 					  &braille->translation_table, 
 					  default_braille_transaltion_table);		
@@ -800,22 +836,22 @@ src_braille_load_values (SRCBraille *braille)
     }
 	
     
-    gchar *default_braille_style = DEFAULT_BRAILLE_STYLE;
+    default_braille_style = DEFAULT_BRAILLE_STYLE;
     GET_BRAILLE_CONFIG_DATA_WITH_DEFAULT (BRAILLE_STYLE, CFGT_STRING, 
 					  &braille->style, 
 					  default_braille_style);
 					  
-    gchar *default_braille_cursor_style = DEFAULT_BRAILLE_CURSOR_STYLE;
+    default_braille_cursor_style = DEFAULT_BRAILLE_CURSOR_STYLE;
     GET_BRAILLE_CONFIG_DATA_WITH_DEFAULT (BRAILLE_CURSOR_STYLE, CFGT_STRING, 
 					  &braille->cursor_style, 
 					  default_braille_cursor_style);
 					  
-    gint default_pos_sensor = DEFAULT_BRAILLE_POSITION_SENSOR;
+    default_pos_sensor = DEFAULT_BRAILLE_POSITION_SENSOR;
     GET_BRAILLE_CONFIG_DATA_WITH_DEFAULT (BRAILLE_POSITION_SENSOR, CFGT_INT, 
 					  &braille->position_sensor, 
 					  &default_pos_sensor);
     
-    gint default_opt_sensor = DEFAULT_BRAILLE_OPTICAL_SENSOR;
+    default_opt_sensor = DEFAULT_BRAILLE_OPTICAL_SENSOR;
     GET_BRAILLE_CONFIG_DATA_WITH_DEFAULT (BRAILLE_OPTICAL_SENSOR, CFGT_INT, 
 					  &braille->optical_sensor, 
 					  &default_opt_sensor);
@@ -840,15 +876,15 @@ src_braille_init ()
     if (!brl_xml_init(src_braille->device, src_braille->port, brl_xml_input_proc)) /* take Braille display from Gnopi... */
     {
 	brlval = FALSE;
-	fprintf(stderr, "\n** BRAILLE initialization failed for %s on port %d",
-					src_braille->device, src_braille->port);
+	fprintf(stderr, "\n** BRAILLE initialization failed for %s on port %s",
+					src_braille->device, src_braille->port ? src_braille->port : "<NONE>");
 	fprintf (stderr, "\n   To change one or more settings use gnopernicus UI\n");
     }
     else
     {
 	brlval = TRUE;
 	fprintf (stderr, "\n** BRAILLE initialization succeded for %s device"
-				    " on port %d.", src_braille->device, src_braille->port);
+				    " on port %s.", src_braille->device, src_braille->port ? src_braille->port : "<NONE>");
 	fprintf (stderr, "\n   To see something you must have a Braille device"
 				    " connected to your computer.");
 	fprintf (stderr, "\n   To change one or more settings use gnopernicus UI.\n");
@@ -875,7 +911,7 @@ src_braille_terminate ()
     brl_in_xml_terminate ();
 	
     g_free (src_braille->device);
-    src_braille->port = -1;
+    g_free (src_braille->port);
 }
 
 static SRCBraille* 
@@ -905,20 +941,47 @@ src_braille_set_device (gchar *device)
 }
 
 gboolean
-src_braille_set_port_no (gint port)
+src_braille_change_port (gchar *nport)
 {
     SRCBraille *braille;
-    
+
     braille = src_braille_get_global_settings ();
     sru_assert (braille);
-    
-    if (braille->port != port)
+
+    if (!nport || !braille->port || strcmp (braille->port, nport) != 0)
     {
-	braille->port = port;
+	g_free (braille->port);
+	braille->port = nport ? g_strdup (nport) : NULL;
 	src_use_braille = src_braille_restart ();
-	return TRUE;
-    }    
-    return FALSE;
+	return src_use_braille;
+    }
+
+    return TRUE;
+}
+
+
+gboolean
+src_braille_set_port ()
+{
+    gboolean rv;
+    gchar *nport = src_braille_get_port ();
+
+    rv = src_braille_change_port (nport);
+    g_free (nport);
+
+    return rv;
+}
+
+gboolean
+src_braille_set_port_no (gint port_no)
+{
+    gboolean rv;
+    gchar *nport = src_braille_get_port_from_no (port_no);
+
+    rv = src_braille_change_port (nport);
+    g_free (nport);
+
+    return rv;
 }
 
 gboolean
@@ -1070,17 +1133,6 @@ src_braille_get_position_sensor ()
     return braille->position_sensor;
 }
 
-gint
-src_braille_get_port_no ()
-{
-    SRCBraille *braille;
-    
-    braille = src_braille_get_global_settings ();
-    sru_assert (braille);
-    
-    return braille->port;
-}
-
 gchar*
 src_braille_get_translation_table ()
 {
