@@ -1,7 +1,7 @@
  /* srspc.c
  *
- * Copyright 2001, 2002 Sun Microsystems, Inc.,
- * Copyright 2001, 2002 BAUM Retec, A.G.
+ * Copyright 2001 - 2005 Sun Microsystems, Inc.,
+ * Copyright 2001 - 2005 BAUM Retec, A.G.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -39,11 +39,17 @@
 #include "unistd.h"
 #include "src-speech-in.h"
 
+#define SRU_DEBUG_MODULE "srcspeech"
+#define SRU_DEBUG_SCOPES ""
+#include "sru-debug.h"
+
 #define STR(X) (X == NULL? "" : X)
 
 static gboolean src_speech_initialized     = FALSE;
 
 static GHashTable *src_dictionary_hash 	= NULL;
+static guint      speech_timeout_id;
+static gboolean   src_speech_restored = FALSE;
 
 typedef struct 
 {
@@ -86,6 +92,7 @@ typedef struct _SRCSOut
 static GPtrArray     	*src_voice_params = NULL;
 SRCSpeech 	 	*src_speech = NULL;
 static SRCVoices 	*src_voices = NULL;
+static GPtrArray	*src_created_voices = NULL;
 
 static gboolean src_speech_idle = TRUE;
 static GSList *srcs_outs = NULL;
@@ -187,14 +194,13 @@ src_speech_valid_voice (gchar *voice)
     gint i;
 
     sru_assert (src_speech_initialized);
-    sru_assert (src_voices->voices);
     sru_assert (voice);
 
-    for (i = 0; src_voices->voices[i]; i++)
+    for (i = 0; i < src_created_voices->len; i++)
     {
-	if (strcmp (voice, src_voices->voices[i]) == 0)
+	if (strcmp (voice, g_ptr_array_index (src_created_voices, i)) == 0)
 	    return TRUE;
-    }    
+    }
     return FALSE;
 }
 
@@ -473,10 +479,10 @@ src_speech_send (gchar *spcoutput)
     if (g_utf8_validate (spcoutput, -1, NULL))
 	srs_output (spcoutput, strlen(spcoutput)); 
     else
-	    sru_warning ("speech output: invalid UTF-8 received");
+        sru_warning ("speech output: invalid UTF-8 received");
     rv = TRUE;
-    
-/*     fprintf (stderr, "\n%s", spcoutput); */
+
+    sru_debug_log ("out", "send to speech:%s", spcoutput);
 
     busy = FALSE;
 
@@ -655,6 +661,7 @@ src_speech_add_voice (gchar *voice)
 	src_voices->voices[i] = voices[i];
     src_voices->voices[i] = g_strdup (voice);
     src_voices->voices[i + 1] = NULL;
+    src_speech_get_voice_parameters_from_gconf (voice);
     g_free (voices);
 
     return TRUE;
@@ -1515,14 +1522,38 @@ src_speech_set_default_values (SRCSpeech *src_speech)
     src_speech->count_mode         = SRC_SPEECH_COUNT_NONE;
 }
 
+static gboolean
+src_timeout_start_speech (gpointer data)
+{
+    gboolean use_speech = TRUE;
+    SET_SRCORE_CONFIG_DATA (SRCORE_SPEECH_ACTIVE, CFGT_BOOL, &use_speech);
+    speech_timeout_id = 0;
+    src_speech_restored = TRUE;
+    return FALSE;    
+}
+
+#define SRC_SPEECH_RESTART_TIMEOUT 1000
+static void
+src_restart_speech ()
+{
+    g_printerr ("\nRestarting speech.\n");
+    gboolean use_speech = FALSE;
+    SET_SRCORE_CONFIG_DATA (SRCORE_SPEECH_ACTIVE, CFGT_BOOL, &use_speech);
+    speech_timeout_id = g_timeout_add (SRC_SPEECH_RESTART_TIMEOUT, src_timeout_start_speech, NULL);
+}
+
 gboolean
 src_speech_init ()
 {
+    gboolean terminate = FALSE;
     sru_assert (!src_speech_initialized);
-    
+
+    SRU_DEBUG_ADD_MODULE_AND_SCOPES;
+
     src_speech_idle = TRUE;
     srcs_outs = NULL;
     last_out = NULL;
+    src_created_voices = g_ptr_array_new ();
     src_voice_params = g_ptr_array_new ();
     
     src_voices = src_speech_src_voices_new ();
@@ -1531,7 +1562,7 @@ src_speech_init ()
     src_speech_set_default_values (src_speech);
     
     srcs_in_init ();
-    src_speech_initialized = srs_init (srcs_in_parse_input);
+    terminate = src_speech_initialized = srs_init (srcs_in_parse_input, src_restart_speech);
 #ifdef SRS_NO_MARKERS_SUPPORTED
     src_speech_callback = TRUE;
     src_speech_unspoken = NULL;
@@ -1548,12 +1579,28 @@ src_speech_init ()
     {
 	src_speech_initialized = src_speech_init_default_voice () &&
 				 src_speech_voices_init ();
+	if (src_speech_initialized)
+	{	
+	    gint i;
+	    src_speech_initialized = FALSE;
+	    for (i = 0; i < src_created_voices->len; i++)
+	    {
+		if (strcmp ("default", g_ptr_array_index (src_created_voices, i)) == 0)
+		{
+		    src_speech_initialized = TRUE;
+		    break;
+		}
+	    }
+	}
     }
     else
     {
 	src_speech_initialized = FALSE;
     }
 
+    if (!src_speech_initialized && terminate)
+	srs_terminate ();
+
     if (src_speech_initialized)
 	src_dictionary_init ();
 
@@ -1570,6 +1617,8 @@ src_speech_init ()
     if (src_speech_initialized)
     {
     	sru_message (_("speech initialization succeded"));
+	if (src_speech_restored)
+	    src_speech_say_message (_("Speech restored"));
     }
     else
     {
@@ -1602,6 +1651,9 @@ src_speech_terminate ()
     g_strfreev (src_voices->voices);
     g_strfreev (src_voices->engine_voices);
     g_strfreev (src_voices->engine_drivers);
+    g_ptr_array_add (src_created_voices, NULL);
+    g_strfreev ((gchar**)g_ptr_array_free (src_created_voices, TRUE));
+    
 }
 
 #define src_speech_is_separator(x) (( x == '.' || x == ';' || x == '?' || x == ',' || x == '!' ) ? TRUE : FALSE)
@@ -1698,11 +1750,11 @@ src_speech_translate_symbol (gint index)
 	    src_speech_dictionary[index].character == '!' ||
 	    src_speech_dictionary[index].character == '?')
 	{
-	    rv = g_strconcat ("  ", src_speech_dictionary[index].singular, " ", NULL);
+	    rv = g_strconcat ("  ", _(src_speech_dictionary[index].singular), " ", NULL);
 	    rv[0] = src_speech_dictionary[index].character;
 	}
 	else
-	    rv = g_strconcat (" ", src_speech_dictionary[index].singular, " ", NULL);
+	    rv = g_strconcat (" ", _(src_speech_dictionary[index].singular), " ", NULL);
     }
     else
     {
@@ -1737,8 +1789,8 @@ src_speech_count_symbol (gchar *start,
 	rv = g_strndup (start, 1);
     else
 	rv = g_strconcat (count, " ",
-	    cnt == 1 ? src_speech_dictionary[index].singular : 
-					src_speech_dictionary[index].plural,
+	    cnt == 1 ? _(src_speech_dictionary[index].singular) : 
+					_(src_speech_dictionary[index].plural),
 	    " ", NULL);
     *new_pos = crt;
     return rv;
@@ -1768,6 +1820,8 @@ src_speech_process_string (const gchar   *text,
 	
     if (text[0] == ' ' && text[1] == '\0' && smode == SRC_SPEECH_SPELL_NORMAL)
 	return g_strdup (_("space"));
+    if (text[0] == '\t' && text[1] == '\0' && smode == SRC_SPEECH_SPELL_NORMAL)
+	return g_strdup (_("tab"));
     if (text[0] == '\n' && text[1] == '\0' && smode == SRC_SPEECH_SPELL_NORMAL)
 	return g_strdup (_("enter"));
 
@@ -1775,7 +1829,7 @@ src_speech_process_string (const gchar   *text,
     {
         gint index = src_speech_get_index_for_symbol (crt[0]);
 	if (index != -1)
-	    return g_strdup (src_speech_dictionary[index].singular);
+	    return g_strdup (_(src_speech_dictionary[index].singular));
 	else
 	    return g_strdup ((gchar*)text);
     }
@@ -2188,7 +2242,9 @@ src_speech_add_created_voice (gchar *name
 #endif
 			    )
 {
-    
+    g_assert (name);
+
+    g_ptr_array_add (src_created_voices, g_strdup (name));
 #ifdef SRS_NO_MARKERS_SUPPORTED
     if (src_speech_callback)
     {
